@@ -483,6 +483,22 @@ class Client(object):
         url = "https://graph.microsoft.com/beta/me/drive/items/{0}/workbook/closeSession".format(item_id)
         return self._post(url, **kwargs)
 
+    @token_required
+    def drive_download_contents(self, item_id, params=None, **kwargs):
+        url = "https://graph.microsoft.com/beta/me/drive/items/{0}/content".format(item_id)
+        return self._get(url, params=params, **kwargs)
+
+    @token_required
+    def drive_get_item(self, item_id, params=None, **kwargs):
+        url = "https://graph.microsoft.com/beta/me/drive/items/{0}".format(item_id)
+        return self._get(url, params=params, **kwargs)
+
+    @token_required
+    def drive_upload_item(self, item_id, params=None, **kwargs):
+        url = "https://graph.microsoft.com/beta/me/drive/items/{0}/content".format(item_id)
+        kwargs['headers'] = {'Content-Type': 'text/plain'}
+        return self._put(url, params=params, **kwargs)
+
     # Excel
     @token_required
     def excel_get_worksheets(self, item_id, params=None, **kwargs):
@@ -600,7 +616,7 @@ class Client(object):
         if 'application/json' in response.headers['Content-Type']:
             r = response.json()
         else:
-            r = response.text
+            r = response.content
         if status_code in (200, 201, 202):
             return r
         elif status_code == 204:
@@ -648,4 +664,9 @@ class Client(object):
         elif status_code == 509:
             raise exceptions.BandwidthLimitExceeded(r)
         else:
+            if r['error']['innerError']['code'] == 'lockMismatch':
+                # File is currently locked due to being open in the web browser
+                # while attempting to reupload a new version to the drive.
+                # Thus temporarily unavailable.
+                raise exceptions.ServiceUnavailable(r)
             raise exceptions.UnknownError(r)
